#################################################
### Deskriptive Datenanalyse
#################################################

setwd("G:\\PMR")
kenya <- read.table("data\\kenya_hh_survey.txt",header=TRUE)

kenya
class(kenya)
summary(kenya)
# Quelle: http://data.worldbank.org/
# -> Surveys in Microdata

rm(list=ls())
str(kenya)
attach(kenya)
detach(kenya)
names(kenya)

### EINE STETIGE Variable: Nonfood Expenditures

# Mittelwert
mean(exp_nonfood_1w)
sum(exp_nonfood_1w)/length(exp_nonfood_1w)

# Minimum und Maximum
min(exp_nonfood_1w) # which.min(exp_nonfood_1w) exp_nonfood_1w[15]
max(exp_nonfood_1w)

#Index von Minimum und Maximum
which.min(exp_nonfood_1w)
which.max(exp_nonfood_1w)

exp_nonfood_1w[801] # numerische Indizierung liefert wieder das Maximum

# Ein Individuum gibt mehr als 10% der anderen Individuen aus
# Wieviel? -> 10% Quantil
quantile( x = exp_nonfood_1w, 
	        probs = c(0.05,0.1,0.2, 0.5, 0.8,0.9,0.95))
?quantile

# Varianz und Standardabweichung
var(exp_nonfood_1w)  
sd(exp_nonfood_1w)  

# Vergleich mit Ausgaben für Essen
mean(exp_food_1w)

quantile(exp_food_1w, probs = c(0.05,0.1,0.2, 0.5, 0.8,0.9,0.95))

min(exp_food_1w)
max(exp_food_1w)
var(exp_food_1w)  # variance
sd(exp_food_1w)  # standard deviation

# Was "summary" macht, hängt von der Klasse ab

class(exp_nonfood_1w)
class(tenure_status_dwelling)
summary(exp_nonfood_1w)
summary(tenure_status_dwelling)


### Aufgabe D1
# Verwenden Sie den Datensatz CPS1985 aus dem AER Paket:
# a) Erstellen Sie nun ein Histogram der logarithmierten Löhne.
# hist
# b) Erweitern Sie dieses um die Dichtefunktion einer normalverteilten Zufallsvariable, 
#    welche Mittelwert und Varianz der logarithmierten Löhne aufweist.
# ?dnorm
plot(dnorm, -2, 2) # d für density
# curve mit ADD
curve(sin, -2, 2)
curve(dnorm, -2, 2, add=TRUE)
#Alternative: lines
# c) Zeichnen Sie den Mittelwert als gestrichelte Linie ein.
abline
# d) Zeichnen Sie mit einem Polygon die 25% Quantile ein.


### MEHRERE STETIGE Zufallsvariablen

# Matrix dreier Ausgabenvariablen
expend <- cbind( exp_nonfood_1w, exp_food_1w, exp_nonfood_6m)
expend


# Kovarianzmatrix: Varianzen auf der Hauptdiagonale, Kovarianzen darunter
var(expend)

# Korrelationen
cor(expend)

# Ist Abhängigkeit linear -> siehe nächste Einheit
plot(x= exp_food_1w, y = exp_nonfood_1w)

which.max(exp_food_1w)
exp_food_1w[which.max(exp_food_1w)]
# Verwerfen 
exp_food_1w     <- exp_food_1w[-which.max(exp_food_1w)]
exp_nonfood_1w<- exp_nonfood_1w[-which.max(exp_food_1w)]

exp_food_1w     <- exp_food_1w[-which.max(exp_nonfood_1w)]
exp_nonfood_1w<- exp_nonfood_1w[-which.max(exp_nonfood_1w)]

# Box Cox 
plot(exp_food_1w,exp_nonfood_1w)

plot.default(log(exp_food_1w),log(exp_nonfood_1w))	# scheint log-linear

expend
# Matrix der log expenditures
lexpend <- log(expend)
lexpend
expend
lexpend[lexpend==-Inf]
# Wert NA für expenditure=0, da log(0)=-Inf

lexpend[lexpend==-Inf] <- NA


# Größere Korrelation
cor(lexpend, use="pairwise.complete.obs")
?cor

### KATEGORIALE Daten
hh_type
class(hh_type)

# Häufigkeiten 
summary(hh_type)
table(hh_type)

# Mehrere kategoriale Variablen
ftable(tenure_status_dwelling,type_dwelling)

ftable(tenure_status_dwelling, type_dwelling, urban_rural)
ftable(tenure_status_dwelling,urban_rural,type_dwelling)
ftable(urban_rural,tenure_status_dwelling,type_dwelling)

# Etwas flexibler
?xtabs	
xtabs(~tenure_status_dwelling+type_dwelling+urban_rural)
xtabs(exp_nonfood_1w~ tenure_status_dwelling+type_dwelling+urban_rural)

# Bedingte Auswertungen
tapply(exp_nonfood_1w, tenure_status_dwelling, mean)


#################################################
# Zufallsvariablen und deren Verteilung
#################################################
?distributions
# Verteilungen und Zufallsstichproben (Übersicht)
?dnorm	# Dichtefunktion der Normalverteilung
?pnorm	# Verteilungsfunktion -"-
?qnorm	# Quantilsfunktion -"-
?rnorm	# Generierung von normalverteilten Zufallsvariablen

norm
t
chisq
f

?dchisq
pchisq
rchisq

# Generierung von Zufallsvariablen
set.seed(42)	# Ein Seed ist wichtig für Reproduzierbarkeit
?rnorm
(zv1 <- rnorm(1000, mean = 3, sd = 1.2))
summary(zv1)
sd(zv1)
dev.off()

hist(zv1, freq = FALSE, breaks=100)
lines(density(zv1))

# Dichte- und Verteilungsfunktionen
probs <- seq(from = 0, to = 1, by = 0.001)

quant <- qnorm(probs, mean = 0, sd = 1)
quant
probs == pnorm(quant, mean = 0, sd = 1)   # Rundungsfehler!

# Darstellung von Dichte und Verteilungsfunktion
plot(quant, dnorm(quant, mean = 0, sd = 1), type = "l", lwd = 2)
plot(quant, probs, type = "l", lwd = 2, main = "Verteilungsfunktion der Std.-Normalverteilung")	
lines(quant, dnorm(quant, mean = 0, sd = 1), type = "l", lwd = 2)

# Weitere Verteilungen
?Distributions

?df

#################################################
### Einfache Hypothesentests
#################################################

# Laden des wheat.txt Datensatzes
setwd("G:/PMR")
wheat <- read.table("data\\wheat.txt",header=TRUE)
wheat

wheat <- na.omit(wheat)
wheat
str(wheat)
attach(wheat)

# Zeitreihenplot
plot.ts(wheat[, 3:4])

# t-Test für Hypothesen über den Erwartungswert
# H0: Der Erwartungswert der Preisänderungen ist kleiner/gleich null -> Kein Aufwärtstrend
mean(Change)


?t.test
t.test(Change,
    alternative = "greater",
    mu = 0,
    conf.level = 0.95)

# t-Test über gleichen Erwartungswert in zwei Samples
# H0: Erw.w ist derselbe 1982-1999 und 2000-2012
t.test(x=Change[Year<2000], y=Change[Year>=2000],
    alternative = "two.sided",
    mu = 0,
    conf.level = 0.95)

# F-test über gleiche Varianz
# H0: Die Varianz 1982-1999 ist dieselbe wie 2000-2012
var.test(x=Change[Year<2000], y=Change[Year>=2000],
        ratio = 1,
        alternative = "less",
        conf.level = 0.95)

# Chi^2-Test auf stochastische Unabhängigkeit zweier kategorialer Variablen
# H0: Die Art der Unterkunft ist unabhängig vom Migrationsstatus
chisq.test(xtabs( ~hh_type+type_dwelling))

ftable(type_dwelling,hh_type)



### Aufgabe D2
# Überfliegen Sie den Aufsatz
# "Incentives Work: Getting Teachers to Come to School" von Duflo et al.
http://economics.mit.edu/faculty/eduflo/papers
# a) Beschreiben Sie kurz das Experiment, das hier durchgeführt wurde.
# b) Schauen Sie sich dann die Daten an  -> Incentives Work Data
#    Die folgenden Datensätze sind ausreichend:
#    TreatmentSchools, Posttest
#    Laden Sie die Daten in R und schauen Sie sie an.
#    Versuchen Sie hierbei den Stata Datensatz (Endung .dta) mit Hilfe der 
#    read.dta Funktion aus dem foreign Package zu laden.
# c) Erstellen Sie im Datensatz Posttest eine Variable "treat", die
#	   den Wert TRUE annimmt für Schüler, deren School-ID (Variable schid) im 
#    Datensatz TreatmentSchools auftaucht. Verwenden Sie den Operator  %in%
#    Beispiel: 
     a <- 1:10
     a[1:10 %in% c(2,3)] 
# d) Erzeugen Sie eine Teilstichprobe des Datensatzes, der die Schüler mit 
#	   schriftlichem Test enthält (post_writ==1)
# e) Vergleichen Sie die mittleren Testergebnisse (Variable post_total_w) 
#    von Treatment- und Nicht-Treatment-Schülern. 
# f) Führen Sie einen t-Test der folgenden Nullhypothese durch:
#    "Treatment- und Nicht-Treatment-Schüler sind im Mittel gleich gut"


### Aufgabe D3*
# Schreiben Sie eine Funktion, welche eine deskriptive statistische Auswertungen
# von Datensätzen beliebiger Struktur erledigt. 



### Aufgabe D4*
# Versuchen Sie die unter D3 geschriebene Funktion gegenüber Fehleingaben weitgehend abzusichern.


