#################################################
### Schätzen
#################################################

### Daten laden: Wachstumsdaten von Stock und Watson (2007)
install.packages("AER")
library(AER)
data(GrowthSW)

?GrowthSW

str(GrowthSW)
summary(GrowthSW)

rownames(GrowthSW)
GrowthSW["Germany",]
GrowthSW["Cyprus",]

attach(GrowthSW)
detach(GrowthSW)
### Ökonomische Fragen:
# - Findet Einkommenskonvergenz statt (führt geringes Einkommensniveau
#   zu c.p. höherem Wachstum in der Zukunft?)
# - Ist Bildung eine wichtige Determinante des Wachstums?
# - Beschleunigt Außenhandel das Wachstum?
# - Ist politische Stabilität wichtig / behindert Gewalt
#   den ökonomischen Wohlstand?
# - Ist ceteris paribus in Afrika das Wachstum langsamer?

### Schätzen eines multiplen Regressionsmodells (lm: linear model) mit OLS
# data=... eigentlich unnötig, da attached!
# y = X beta + u mit y n x 1, X n x k Matrix, beta k x 1 , u nx1 
# Struktur bzw. Formula: 
# Schätzname <- linear model (Regressand/y ~ Regressoren/x + Konstante)
?lm

# was ist eine Formel in R?
?formula

growth.eq1 <- lm(growth ~ rgdp60, 
                 data=GrowthSW)


growth.eq1 <- lm(growth ~ rgdp60+tradeshare+education+revolutions+assassinations, 
	data=GrowthSW)

class(growth.eq1) # lm Klasse

str(growth.eq1, max=1) # 1. Ebene

summary(growth.eq1)

attach(GrowthSW)
growth.eq1 <- lm(growth ~ rgdp60 + tradeshare + education +
                   revolutions + assassinations)

class(growth.eq1)           # lm für linear model (wieder ein neues Objekt!)


# Standard-Output
summary.eq1 <- summary(growth.eq1)
summary.eq1

str(growth.eq1, max=1)
?str

# Extrahieren von Residuen, gefitteten Werten usw.
(u_hat <- resid(growth.eq1))  
growth.eq1$residuals
# oder 
growth.eq1$rank
growth.eq1

growth.eq1$coefficients
class(growth.eq1)

(u_hat <- growth.eq1$resid)
y_hat <- fitted(growth.eq1)
y_hat <- growth.eq1$fitted


sqrt(sum(u_hat^2)/(length(rgdp60)-6))




# Das summary-Objekt hat mehr zu bieten:
summary.eq1 <- summary(growth.eq1)
str(summary.eq1, max=1)
summary.eq1$coefficients

# Ansprechen wie bei Matrizen, 5 . Zeile, 2. Spalte
summary.eq1$coefficients[5,2]
summary.eq1$r.squared

summary.eq1$adj.r.squared


### Dummies, Faktoren, Quadratische Terme, Interaktionen usw.
growth.eq1 <- lm(growth~rgdp60+tradeshare+education+revolutions+assassinations)
summary(growth.eq1)

# ohne Konstante
growth.eq1_noconstant <- lm(growth~-1+rgdp60+tradeshare+education+revolutions+assassinations)
summary(growth.eq1_noconstant)

# Erstelle "character"-Vektor continent: africa oder other
GrowthSW$continent <- "other"
summary(GrowthSW)
GrowthSW$continent

# Ansprechen der continent Spalte mit den folgenden Ländenr
GrowthSW[c("Zaire","Niger","Senegal", "Zimbabwe" ,"Togo",
            "South Africa", "Sierra Leone" ,"Ghana","Kenya",
            "Mauritius"),"continent"]   <- "africa"
GrowthSW$continent

a <- c("Zaire","Niger","Senegal", "Zimbabwe" ,"Togo",
       "South Africa", "Sierra Leone" ,"Ghana","Kenya","Mauritius")
b <- "Zaire"
a %in% b

class(GrowthSW$continent)

# Reihenfolge der levels bestimmt Referenzkategorie
(GrowthSW$continent <- factor(GrowthSW$continent,levels=c("other","africa")))
summary(GrowthSW)

# Ein factor in der Formel führt zur Expansion in einzelne Dummies (auch für mehrere Kategorien!)
growth.eq2 <- lm(growth~rgdp60+tradeshare+education+revolutions+assassinations+continent,
	data=GrowthSW)

summary(growth.eq2)

# Oder: Regression auf ALLE Variablen im Datensatz mit "."
summary( lm(growth ~ . , data=GrowthSW)  )

# Versuche nichtlineare Terme in x: Quadratisch, Logarithmiert, ...
# Beachte: Bei Transformationen von Regressoren (x1+x2, x3^2, x4/5, ...)
# die I() Funktion verwenden; 
growth.eq3 <- lm(growth~rgdp60 + log(rgdp60) +tradeshare+education+log(education)+
	revolutions+ I(revolutions^2) +assassinations+continent, data=GrowthSW)


summary(growth.eq3)

# Erlaube unterschiedliche Konvergenzgeschwindigkeit für Afrika
# -> Interaktion rgdp60 mit continent
growth.eq4 <- lm(growth~
	log(rgdp60)*continent +
	tradeshare+log(education)+
	revolutions+assassinations, data=GrowthSW)
summary(growth.eq4)

# Alle Interaktionen einer Regressorgruppe einfügen (in Klammernquadrat)
growth.eq5 <- lm(growth~ (log(rgdp60)+tradeshare+log(education))^2 +
                        revolutions+assassinations+continent, data=GrowthSW)
summary(growth.eq5)

# Konstante und Regressoren entfernen mit "-"
growth.eq6 <- lm(growth~ -1+
	(log(rgdp60)+tradeshare+log(education))^2+revolutions+assassinations+continent - 
	log(rgdp60):log(education), data=GrowthSW)
summary(growth.eq6)



### Modellwahl

# Informationskriterien: AIC, SC, ...
# Ist Modell 4 "besser" als Modell 1?

# Akaike Kriterium
AIC(growth.eq5)     # kleiner = besser
AIC(growth.eq1)

# Schwarz Kriterium
BIC(growth.eq5)     # besser
BIC(growth.eq1)

# Schrittweise Suche mit AIC, ausgehend von sehr allgemeinem Modell (alle Interaktionen)
install.packages("MASS")
library(MASS)
?stepAIC
growth.eqAIC <- stepAIC(lm(growth~(log(rgdp60)+tradeshare+log(education)+
	revolutions+assassinations+continent)^2, data=GrowthSW))
class(growth.eqAIC)
sum<- summary(growth.eqAIC)

class()
str(sum, max=1)

### Aufgabe F1
# Nutzen Sie den Growth Datensatz von oben.
# a) Testen Sie ein paar Möglichkeiten für die 
#    Formeleingabe:
#    - Ohne/mit Konstante
#    - Quadratisch, Kubisch, den doppelten Regressor
#    - Interaktionen (von Gruppen)
#    - stepAIC Befehl benutzen
# b) Nehmen Sie nun eines Ihrer Modelle und wenden 
#    Sie den summary()-Befehl darauf an. Sprechen Sie
#    Schätzungen, Standardfehler, p-values, die Residuenquadratsumme
#    und den angepassten R^2 Wert an. 

summary(lm(growth ~ I(education^3)))
summary(lm(growth ~ education^3))
summary(lm(growth ~ (education+revolutions+rgdp60)^3))

#################################################
### Diagnose und Testen
#################################################

### Modelldiagnose: 1. Heteroskedastie

# Graphisch
uhat_AIC <- resid(growth.eqAIC) # growth.eqAIC$residuals
yhat_AIC <- fitted(growth.eqAIC)
plot(yhat_AIC,uhat_AIC^2,main="Squared Residuals vs. Fitted Values")   # Scheint problematisch

# White Test
summary(lm(I(uhat_AIC^2)~yhat_AIC))  				# Heteroskedastie
# Breusch - Pagan
summary(lm(I(uhat_AIC^2)~.-growth, data=GrowthSW))	# Heteroskedastie
bptest(growth.eqAIC)								  # ... oder nicht?
# Unterschied: BP-Test verwendet Chi^2-Statistik

# Heteroskedastie-robuste Varianz-Kovarianzmatrix (eigentlich immer benutzen!)
?vcovHC
vcov.robust <- vcovHC(growth.eqAIC, type = "HC") 
coeftest(growth.eqAIC,vcov=vcov.robust)

summary(growth.eqAIC)

### Modelldiagnose: 2. Nichtlinearität

# Anzeichen für Nichtlinearität?
plot(yhat_AIC,uhat_AIC,main="Residuals vs. Fitted Values")  
# RESET-Test
resettest(growth.eqAIC)

# Einige Plots (Ausreißer usw.)
class(growth.eqAIC)
plot(growth.eqAIC)
plot(growth.eq1)

summary(growth.eq1)
### Hypothesentests einer einzelnen Hypothese: t-Tests

# H_0: beta_j = 0   ->   aus dem Output ablesen!

# H_0: beta_j = 'beliebiger Wert', etwa
# H_0: beta_educ = 2 ("Ein Jahr längeren Schulbesuch erhöht W'tum um 2 Proz.punkte")

growth.eqAIC

# Manuell:
( coef_aic <- coefficients(summary(growth.eqAIC)) )
t_stat_educ <- ( coef_aic["log(education)","Estimate"] - 2 ) / coef_aic["log(education)","Std. Error"]
t_stat_educ
p_value <- 2* pt(t_stat_educ, df= nrow(GrowthSW)-nrow(coef_aic) )    # df=n-k, 2*... wegen beidseitigem Test
p_value         # Nicht ablehnen auf 0.05 Niveau

# Einfacher: mit Paket 'lmtest' (wird mit AER mitgeladen)
library(lmtest)

# Verbal: (Vorsicht: Gemeint ist der KOEFFIZIENT VOR log(education) !!!!)
?linearHypothesis
linearHypothesis(growth.eqAIC, "log(education)=2")

# Andere Form der Hypothese: R%*%beta=r  (hier ist R ein 1xk-Vektor, r skalar)
R <- c(0,0,0,1,0,0,0,0,0)   # 4. Eintrag wählt 4. Koeffizienten
r <- 2                      # .. und der soll 2 sein laut H_0
linearHypothesis( growth.eqAIC , hypothesis.matrix =R, rhs=r )

# Heteroskedastie-Robust:
linearHypothesis(growth.eqAIC,"log(education)=2",vcov=vcov.robust)


### Tests von mehreren Hypothesen: F-Tests

# H_0: beta_log(rgdp60) = 0  UND  beta_log(rgdp60):assassinations = 0
# ("Das Anfangseinkommen hat keine Auswirkung auf das Wachstum" bzw. "Keine Konvergenz")

# Das restringierte Modell ist
growth.eqRESTR <- update(growth.eqAIC,.~.-log(rgdp60)-log(rgdp60):assassinations )
summary(growth.eqRESTR)

# Möglichkeit 1: 
anova(growth.eqAIC,growth.eqRESTR)

# Möglichkeit 2:
linearHypothesis(growth.eqAIC,c("log(rgdp60)=0","log(rgdp60):assassinations=0"))
summary(growth.eqAIC)


# Möglichkeit 3: 
R <- matrix(c(0,1,rep(0,7), rep(0,7),1,0),nrow=2, byrow=TRUE)
r <- c(0,0)
linearHypothesis(growth.eqAIC,R,r)

# Heteroskedastie-Robust:
linearHypothesis(growth.eqAIC,c("log(rgdp60)=0","log(rgdp60):assassinations=0"),vcov=vcov.robust)


### Konfidenzintervalle
confint(growth.eqAIC,parm=c("revolutions"),level=0.9)



#################################################
### Prognose
#################################################

# Neue Beobachtungen
X0 <- data.frame( growth=c(NA,NA),
                  revolutions=c(0.3,0.13),
                  rgdp60=c(2000,6000),
                  tradeshare=c(0.3,0.7),
                  continent=factor(c("africa","other"),levels=c("other","africa")), 
                  assassinations=c(0.4,0.2),
                  education=c(2,4)  )
                  
rownames(X0) <- c("Land 1", "Land 2")
 
# Punktprognose, Prognoseintervall, etc. 
?predict
predict(growth.eqAIC,newdata=X0,se.fit=TRUE,interval="prediction")



#################################################
### FGLS bei Heteroskedastie
#################################################

# Erzeuge (inverse) Gewichte (geschätzte Varianz des Fehlers einer Beobachtung)
var.eq <- lm(log(uhat_AIC^2)~.-growth,data=GrowthSW)
summary(var.eq)
sigma_sq_hat <- exp(fitted(var.eq))

# Gewichte mit "weights"-Argument verwenden
growth.eqFGLS <- update(growth.eqAIC,weights=1/sigma_sq_hat)
summary(growth.eqAIC)
summary(growth.eqFGLS)

# Ergebnisvergleich
# Standard OLS
summary(growth.eqAIC)
# Heteroskedastierobuste Standardfehler
coeftest(growth.eqAIC,vcov=vcov.robust)
# FGLS
summary(growth.eqFGLS)

# -> Interpretation !!!

### Aufgabe F2
# a) Benutzen Sie die Daten 'CPS1985' (siehe D_Data_Analysis.R)
#    um ein ad-hoc Regressionsmodell für den logarithmierten 
#    Stundenlohn zu spezifizieren und zu schätzen. 
# b) Führen Sie nun ausgehend von einem sehr allgemeinen Modell
#    eine Modellwahl durch, etwa durch jew. Entfernen des am wenigsten signifikanten
#    Regressors. Testen Sie danach die dabei verwendeten Ausschlussrestriktionen.
#    Hinweise:
#    - Allgemeines Modell: Formel in Quadrat für alle möglichen Interaktionen
#    - Schleife mit Abfrage, ob der maximale p-value größer als 5% ist. 
#    - Löschen dieses Regressors aus dem Modell. Wichtig: Ausgabe der Matrix X im Modell mit 
#      lm(y~x, x=TRUE), da dann das Löschen des jeweiligen Regressors einfacher ist.
#      Die Designmatrix X kann dann über a$x, wobei a <- lm(y~x, x=TRUE), aufgerufen werden. 
#    - Alternativ: Einfügen einer neuen Formel, Problem aber bei Interaktionen. 
# c) Versuchen Sie nun die beiden Modelle anhand geeigneter Diagnosetests, Kriterien 
#    Grafiken etc. zu vergleichen. Entscheiden Sie sich für ein Modell.
# d) Welchen Lohn hätte derjenige Angestellte zu erwarten, der bezüglich aller erklärenden
#    Charakteristika dem Median entspricht. Welchen der "Mittelwertarbeitnehmer"?
# e) Geben Sie jeweils ein Prognoseintervall an.   

#################################################
### Dynamische Regression
#################################################

install.packages(c("MASS","lmtest","dynlm"),repos="http://cran.rakanu.com")
library(MASS)
library(lmtest)
install.packages("dynlm")
library(dynlm)

?dynlm


# Beispiel "Phillipskurve"
# AUFGABE:

# Laden Sie (erneut) die US-Makrodaten
# (Datensatz data.us siehe Aufgabe C.2) in R ein.


setwd("G:/PMR")
data.us <- read.csv("data\\dataUS.csv")[,-1]
data.us
data.us[,1] <- as.Date(data.us[,1])
summary(data.us)
n <- nrow(data.us)


# Schätzen Sie eine Phillipskurve, d.h. eine Regression
# der Arbeitslosenquote auf die Quartalsinflationsrate.
# Prüfen Sie die Validität der Spezifikation. Interpretieren
# Sie, falls möglich, das Ergebnis.

plot(inf~ur, data=data.us, main="Scatterplot")
plot.ts(data.us[,c("inf","ur")], main="Zeitreihenplot")

est.pk <- lm(inf~ur,data=data.us)
summary(est.pk) # Interpretation...

# Residuen anschauen
resid.pk <- est.pk$resid
plot.ts(resid.pk)

plot(resid.pk[-length(resid.pk)],resid.pk[-1],xlab="u(t-1)",ylab="u(t)",main="Autokorrelation")


# Fügen Sie nun auch VERZÖGERTE Inflation und Arbeitslosigkeit
# in das Modell ein. Spezifizieren Sie ein möglichst "gutes" Modell.
# Interpretation...

u <- ts(data.us$ur, start=c(1948,1), frequency=4)
inf <- ts(data.us$inf, start=c(1948,1), frequency=4)

PhilData <- ts.intersect(   u = u, u.l1 = lag(u,-1),
                            u.l2  = lag(u,-2), u.l3  = lag(u,-3),
                            u.l4  = lag(u,-4),
                            inf = inf, inf.l1 = lag(inf,-1),
                            inf.l2 = lag(inf,-2), inf.l3 = lag(inf,-3),
                            inf.l4 = lag(inf,-4), dframe=TRUE )

# Schätzung des allgemeinen Modells
PC_gen <- lm(inf~.,data=PhilData)
summary(PC_gen)
# Und Modellwahl mit AIC
PC_AIC <- stepAIC(PC_gen)
summary(PC_AIC)

# Autokorrelation ?
uhat_PC <- resid(PC_AIC)
plot.ts(uhat_PC)
acf(uhat_PC)
dwtest(PC_AIC)   # Hier unzulässig!
bgtest(PC_AIC,order=8)
Box.test(uhat_PC,lag=8,type="Box-Pierce", fitdf=PC_AIC$rank)



### Mit embed()
PhilData <- as.matrix( subset(data.us,select=c("inf","ur")) )
PhilData <- embed(PhilData,5)
PhilData <- as.data.frame(PhilData)
names(PhilData) <- c("inf","u" ,paste(c("inf.l","u.l"),rep(1:4,each=2),sep=""))
summary(PhilData)


### Mit dynlm

PhilData <-ts( subset(data.us,select=c("inf","ur")), start=c(1948,1), frequency=4)

plot(PhilData)

PC_AIC <- dynlm( inf ~ ur + L(ur,3) + L(inf,1:3) ,data=PhilData)
summary(PC_AIC)


### Aufgabe F3
# a) Laden Sie den dataUS-Datensatz.
# b) Schätzen Sie eine Phillipskurve, d.h. eine Regression
# 	 der Quartals-Inflationsrate auf die Arbeitslosenquote.
# 	 Prüfen Sie die Validität der Spezifikation. Interpretieren
# 	 Sie, falls möglich, das Ergebnis.
# c) Fügen Sie nun auch VERZÖGERTE Inflation und Arbeitslosigkeit
# 	 in das Modell ein. Spezifizieren Sie ein möglichst "gutes" Modell.
# 	 Interpretation...


    







